# -*- coding: UTF-8 -*-
# Copyright 2015-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
# todo update doc string
"""A readonly user interfaces to :mod:`lino_book.projects.team`.
See :ref:`noi.specs.bs3`.

.. autosummary::
   :toctree:

   settings
   tests

"""

