"""
.. autosummary::
   :toctree:

    openui5
    projects

"""


from .setup_info import SETUP_INFO

__version__ = SETUP_INFO['version']

# intersphinx_urls = dict(docs="http://openui5.lino-framework.org")
srcref_url = 'https://github.com/lino-framework/openui5/blob/master/%s'
# doc_trees = ['docs']
